/*
 * Copyright (c) 2015, 2016, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package com.oracle.truffle.r.runtime.ffi.jni;

import com.oracle.truffle.r.runtime.RInternalError;
import com.oracle.truffle.r.runtime.ffi.LibPaths;
import com.oracle.truffle.r.runtime.ffi.PCRERFFI;

public class JNI_PCRE implements PCRERFFI {
    JNI_PCRE() {
        System.load(LibPaths.getBuiltinLibPath("pcre"));
    }

    @Override
    public long maketables() {
        return nativeMaketables();
    }

    @Override
    public Result compile(String pattern, int options, long tables) {
        return nativeCompile(pattern, options, tables);
    }

    @Override
    public Result study(long code, int options) {
        throw RInternalError.unimplemented("pcre_study");
    }

    @Override
    public int exec(long code, long extra, String subject, int offset, int options, int[] ovector) {
        return nativeExec(code, extra, subject, offset, options, ovector, ovector.length);
    }

    private static native long nativeMaketables();

    private static native Result nativeCompile(String pattern, int options, long tables);

    private static native int nativeExec(long code, long extra, String subject, int offset,
                    int options, int[] ovector, int ovectorLen);

}
